clear; clc;

% Load GDP and debt data

load data_BOE YBHA3 debt_boe % GDP is 1700-2017, Debt is 1691-2016

% Load Quantity data (Q3 and Q4 contain different gilts)

Q1 = dataset('XLSFile','Q_1694m3-1860m2.xlsx');
Q2 = dataset('XLSFile','Q_1860m3-1881m2.xlsx');
Q3 = dataset('XLSFile','Q_1881m1-1947m12.xlsx');
Q4 = dataset('XLSFile','Q_1900m11-1947m12.xlsx');
Q5 = dataset('XLSFile','Q_1946m1-2018m12.xlsx');
Q=[Q1;Q2(Q2.Year<1881,:);Q3;Q4;Q5(Q5.Year>1947,:)];

% Load characteristics of individual gilts

d = dataset('XLSFile','Specification.xlsx');

% Define period of interest

startyear=1700;
endyear=2017;

% Define variables to store results

Maturity_y=zeros(endyear-startyear+1,1);
N=zeros(endyear-startyear+1,1);
Ni=zeros(endyear-startyear+1,1);
Nr=zeros(endyear-startyear+1,1);
Qtot=zeros(endyear-startyear+1,1);

% Loop through years

for year=startyear:endyear
    
    year
              
    % Store year
        
    index_year(year-startyear+1,:)=year;
        
    % Select gilts for which quantity>0 in December
        
    % Non index-linked gilts (some anomalies omitted)
    Qtemp1=Q(Q.Year==year & Q.Month==12 & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
    & Q.ID~=2300 & Q.ID~=2700 ...
    & Q.ID~=31200 & Q.ID~=30800 & Q.ID~=26800 & Q.ID~=31000 ...
    & Q.ID~=28100 & Q.ID~=17300 & Q.ID~=15700 & Q.ID~=14400 ...
    & Q.ID~=13300 & Q.ID~=12700 & Q.ID~=13000 ...
    & Q.ID~=18400 & Q.ID~=19600 & Q.ID~=31800 ...
    & Q.ID<32300,:);
    % Index-linked gilts   
    Qtemp2=Q(Q.Year==year & Q.Month==12 & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
    & Q.ID>49999 & Q.ID<100000,:);
    % Consols
    Qtemp3=Q(Q.Year==year & Q.Month==12 & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
    & Q.ID>32299 & Q.ID<50000,:);
    % Miscellaneous old Consols
    Qtemp4=Q(Q.Year==year & Q.Month==12 & Q.Coupon>0 & Q.ID>0 & Q.Q>0 ...
    & Q.ID>=100000,:);
    
    % Save gilts 
    Qtemp=[Qtemp1;Qtemp2;Qtemp3;Qtemp4];

    % Define variables to store results
        
    n=size(Qtemp,1);
    Qs=zeros(n,1);
    startQ=zeros(n,1);
    endQ=zeros(n,1);
              
    % Loop through gilts to see if issued or retired during year
        
    ID=Qtemp.ID;
                
    for i=1:n;
        % Quantity of gilt outstanding
        Qs(i)=Qtemp.Q(Qtemp.ID==ID(i),:);
        % Look up year of issuance
        startQ(i,:)=d.FirstYearQ(d.ID==ID(i),:);
        % Look up year of redemption
        endQ(i,:)=d.LastYearQ(d.ID==ID(i),:);
    end

    % Number of gilts outstanding
    N(year-startyear+1)=n;                  
    % Number of gilts issued
    Ni(year-startyear+1)=sum(startQ==year);
    % Number of gilts retired
    Nr(year-startyear+1)=sum(endQ==year+1);
    % Quantity of outstanding gilts relative to GDP
    Qtot(year-startyear+1)=sum(Qs)/YBHA3(year-startyear+1);
    
end

% Figure 2: Debt instruments

figure(2);
subplot(3,1,1);
plot(index_year,N','LineWidth',2);
title('(a) Number of gilts outstanding','FontSize',24);
set(gca,'FontSize',24);
ylim([0 120]);
xlim([1680 2020]);

subplot(3,1,2);
plot(index_year,[Ni [0;Nr(1:end-1)]]','LineWidth',2);
title('(b) Number of gilts issued and retired','FontSize',24);
legend('Issued','Retired','Location','northwest');
set(gca,'FontSize',24);
xlim([1680 2020]);

subplot(3,1,3);
plot(index_year,(Qtot./(N)),'LineWidth',2);
title('(c) Average size of each outstanding gilt relative to GDP','FontSize',24);
set(gca,'FontSize',24);
xlim([1680 2020]);

