clear;clc;clf

global Yields Prices YBHA3 YBHA3_real debt_boe startyearGDP endyearGDP;
global Qmonthi startyeari endyeari;

% Load yields and prices for counterfactuals

load ZCY_data.mat Yields Yields_r Prices nominal_index real_index

% Load GDP and debt data

load data_BOE YBHA3 YBHA3_real debt_boe % GDP is 1700-2017, Debt is 1691-2016
startyearGDP=1700; endyearGDP=2017;

% Load issuance data

load issuance_f_data Qmonthi startyeari endyeari;

% Define variables for storage

nyears=size(years,2);
store_data=zeros(nyears,nyears);
store_36=zeros(nyears,nyears);
store_60=zeros(nyears,nyears);
store_120=zeros(nyears,nyears);
store_min=zeros(nyears,nyears);

% Set skip=1 for most detailed plot

skip=1;

% Loop through combinations of start and end years

for ystart=1910:skip:2016
    
    for yend=max(ystart,1920)+1:skip:2017 

        [ystart yend]
      
        year1=ystart;
        year2=yend;
    
        % Reset nominal payments on nominal fixed-term debt at monthly frequency

        load returns_f_data index_for_pay pay pay1 pay2;
 
        % Reset face and market values of debt at monthly frequency

        load debt_f_data debt_face debt_face_n debt_face_i debt_market ...
        debt_market_n debt_market_i startyear endyear;

        % Decompose counterfactual

        decompose;

    	% Store results and contribution of new debt in data
       
        fund_fix=Results(9);
        store_data(ystart-1909,yend-1909)=Results(2);

        % 3 year counterfactual

        fixed1([36,ystart,0]);
    
        % Correct market value of debt to include spare cash in counterfactual
    
        load tempsave1 debt_c cash cash_c;
        Total_debt_mv_c=sum(0.01*debt_c.*Prices,2);
        Total_debt_mv_c=Total_debt_mv_c+((cash_c(1:(endyeari-startyeari+1)*12)-cash(1:(endyeari-startyeari+1)*12))...
            .*((cash_c(1:(endyeari-startyeari+1)*12)-cash(1:(endyeari-startyeari+1)*12))<0));

        % Overwrite data for decompositions
    
        pay2=pay;
        pay1=0.*pay1;
        debt_market(end-12*(endyeari-startyeari)+1:end)=Total_debt_mv_c(1:end-12);
        debt_market_n(end-12*(endyeari-startyeari)+1:end)=Total_debt_mv_c(1:end-12);
       
        % Hall-Sargent decomposition on counterfactual data
 
        decompose
        
        % Rescale contribution of nominal return so deficit term is constant across counterfactuals
    
        Results(4)=Results(4)-(fund_fix-Results(9));
        Results(6)=Results(6)-(fund_fix-Results(9));
        Results(9)=Results(9)+(fund_fix-Results(9));
        
        % Store results
    
        store_36(ystart-1909,yend-1909)=Results(2);
    
        % 5 year counterfactuals

        fixed1([60,ystart,0]);
    
        % Correct market value of debt to include spare cash in counterfactual
    
        load tempsave1 debt_c cash cash_c;
        Total_debt_mv_c=sum(0.01*debt_c.*Prices,2);
        Total_debt_mv_c=Total_debt_mv_c+((cash_c(1:(endyeari-startyeari+1)*12)-cash(1:(endyeari-startyeari+1)*12))...
            .*((cash_c(1:(endyeari-startyeari+1)*12)-cash(1:(endyeari-startyeari+1)*12))<0));

        % Overwrite data for decompositions
    
        pay2=pay;
        pay1=0.*pay1;
        debt_market(end-12*(endyeari-startyeari)+1:end)=Total_debt_mv_c(1:end-12);
        debt_market_n(end-12*(endyeari-startyeari)+1:end)=Total_debt_mv_c(1:end-12);
    
        % Hall-Sargent decomposition on counterfactual data
 
        decompose
    
        % Rescale contribution of nominal return so deficit term is constant across counterfactuals
    
        Results(4)=Results(4)-(fund_fix-Results(9));
        Results(6)=Results(6)-(fund_fix-Results(9));
        Results(9)=Results(9)+(fund_fix-Results(9));
    
        % Store results
    
        store_60(ystart-1909,yend-1909)=Results(2);
    
        % 10 year counterfactuals

        fixed1([120,ystart,0]);
    
        % Correct market value of debt to include spare cash in counterfactual
    
        load tempsave1 debt_c cash cash_c;
        Total_debt_mv_c=sum(0.01*debt_c.*Prices,2);
        Total_debt_mv_c=Total_debt_mv_c+((cash_c(1:(endyeari-startyeari+1)*12)-cash(1:(endyeari-startyeari+1)*12))...
            .*((cash_c(1:(endyeari-startyeari+1)*12)-cash(1:(endyeari-startyeari+1)*12))<0));

        % Overwrite data for decompositions.
    
        pay2=pay;
        pay1=0.*pay1;
        debt_market(end-12*(endyeari-startyeari)+1:end)=Total_debt_mv_c(1:end-12);
        debt_market_n(end-12*(endyeari-startyeari)+1:end)=Total_debt_mv_c(1:end-12);

        decompose
    
        % Rescale contribution of nominal return so deficit term is constant across counterfactuals
    
        Results(4)=Results(4)-(fund_fix-Results(9));
        Results(6)=Results(6)-(fund_fix-Results(9));
        Results(9)=Results(9)+(fund_fix-Results(9));
    
        % Store results
    
        store_120(ystart-1909,yend-1909)=Results(2);
        
        % Pick out maturity that minimises average debt
    
        [M,I]=min([store_36(ystart-1909,yend-1909);store_60(ystart-1909,yend-1909);store_120(ystart-1909,yend-1909)]);
        store_min(ystart-1909,yend-1909)=I;

        % Figure 9: Comparison of counterfactuals
    
        figure(9);
        xlim([1910 2020]);
        ylim([1910 2020]);
        xlabel('Start year');
        ylabel('End year');
        set(gca,'FontSize',16);
        if store_min(ystart-1909,yend-1909)==1
            plot(ystart,yend,'r.','MarkerSize',20);
        end
        if store_min(ystart-1909,yend-1909)==2
            plot(ystart,yend,'g.','MarkerSize',20);
        end
        if store_min(ystart-1909,yend-1909)==3
            plot(ystart,yend,'b.','MarkerSize',20);
        end
        hold on;
        drawnow;
        
    end
    
end

% Save results

% save map_temp store_data store_36 store_60 store_120 store_min;